@@ -1,12 +1,12 @@
 package SPOPS::Configure::DBI;
 
-# $Header: /usr/local/cvsdocs/SPOPS/SPOPS/Configure/DBI.pm,v 1.29 2000/10/27 04:05:45 cwinters Exp $
+# $Id: DBI.pm,v 1.30 2000/11/18 21:09:05 cwinters Exp $
 
 use strict;
 use SPOPS::Configure;
 
 @SPOPS::Configure::DBI::ISA     = qw( SPOPS::Configure );
-$SPOPS::Configure::DBI::VERSION = sprintf("%d.%02d", q$Revision: 1.29 $ =~ /(\d+)\.(\d+)/);
+$SPOPS::Configure::DBI::VERSION = sprintf("%d.%02d", q$Revision: 1.30 $ =~ /(\d+)\.(\d+)/);
 
 use constant DEBUG => 0;
 
@@ -272,7 +272,7 @@ it under the same terms as Perl itself.
 
 =head1 AUTHORS
 
-Chris Winters  <cwinters@intes.net>
+Chris Winters  <chris@cwinters.com>
 
 
 =cut
