@@ -1,11 +1,11 @@
 package SPOPS::DBI::MySQL;
 
-# $Header: /usr/local/cvsdocs/SPOPS/SPOPS/DBI/MySQL.pm,v 1.13 2000/10/27 04:05:45 cwinters Exp $
+# $Id: MySQL.pm,v 1.14 2000/11/18 21:09:05 cwinters Exp $
 
 use strict;
 
 @SPOPS::DBI::MySQL::ISA     = ();
-$SPOPS::DBI::MySQL::VERSION = sprintf("%d.%02d", q$Revision: 1.13 $ =~ /(\d+)\.(\d+)/);
+$SPOPS::DBI::MySQL::VERSION = sprintf("%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/);
 
 use constant DEBUG => 0;
 
@@ -73,7 +73,7 @@ it under the same terms as Perl itself.
 
 =head1 AUTHORS
 
-Chris Winters  <cwinters@intes.net>
+Chris Winters  <chris@cwinters.com>
 
 
 =cut
